'''Store the classes that are needed to manage the storage database and
related functions such as searching.'''
import json
import hashlib

import libs.events
import libs.errors

class Block(object):
    '''A class which is used to store data. It is currently limited to 2048
    bits because of the current RSA crypto restrictions.
    '''
    max_size = 2048 # 2048 for now because of the RSA Crypto limit
    def __init__(self, data):
        if len(data) > self.max_size:
            raise libs.errors.AnonError('Block too big.')
        self.data = data
        self.hash = hashlib.sha256(data).hexdigest()
        # TODO: Decide on using sha256 or sha1 (or even sha512)^
        # TODO: Store a last-access timestamp

class StorageDB(libs.events.Handler):
    '''An database type object to store and sort blocks.'''
    def __init__(self):
        self.usks = {}
        self.uuks = {}

    # Database methods
    def add_usk(self, block):
        '''Add a USK block to the data store.'''
        self.usks[block.hash] = block

    def add_uuk(self, block):
        '''Add a UUK block to the data store.'''
        self.uuks[block.hash] = block

    def search(self, query):
        '''Search the database for a block which matches the query.'''
        if query.type == 'UUK':
            if query.id in self.uuks:
                return self.uuks[query.id]
        elif query.type == 'USK':
            if query.id in self.usks:
                return self.usks[query.id]

    def save(self, path):
        '''Save the database from a storage file.'''
        out_json = {}
        for block in self.uuks.values():
            out_json[block.hash] = block.data

        db_file = open(path, 'w') # TODO: possibly make a backup?
        db_file.write(json.dumps(out_json, indent = 2))
        db_file.close()

        libs.events.broadcast('logthis', 'Saved the data store.')

    def load(self, path):
        '''Load the database to a storage file.'''
        db_file = open(path, 'r')
        db_json = json.loads(db_file.read())
        db_file.close

        for block in db_json:
            self.uuks[block] = db_json[block]

    # Event methods
    def got_message(self, packet):
        '''Got a message. For now we are assuming that the message needs to be
        stored but this is not necessarily the case. In future versions we will
        have a custom packet for messages which need to be stored by other
        nodes. By default we are using UUK and there is a 100% chance that we
        will store this packet. There is no file size limit.
        '''
        message = packet.message
        self.add_uuk(Block(message))

    def got_request(self, query):
        '''Got a request for a data block.'''
        self.search(query)

    def shutdown(self):
        '''Save the data store into a file.'''
        self.save(self.path)

class Query(object):
    '''Contain the query parameters and assist in efficient searching of the
    network for the requested data.'''
    def __init__(self, blocktype, id):
        self.type = blocktype
        self.id = id